@@ -22,6 +22,7 @@ from pymarc.exceptions import (
     NoFieldsFound,
     RecordDirectoryInvalid,
     RecordLeaderInvalid,
+    TruncatedRecord,
 )
 from pymarc.field import (
     END_OF_FIELD,
@@ -271,6 +272,8 @@ class Record:
             raise BaseAddressNotFound
         if base_address >= len(marc):
             raise BaseAddressInvalid
+        if len(marc) < int(self.leader[:5]):
+            raise TruncatedRecord
 
         # extract directory, base_address-1 is used since the
         # director ends with an END_OF_FIELD byte
@@ -511,27 +514,18 @@ class Record:
         can be found at the following URL:
         https://www.fdlp.gov/file-repository/gpo-cataloging/1172-gpo-classification-manual
         """
-        if self["086"]:
-            return self["086"].format_field()
-        return None
+        field = self["086"]
+        return field.format_field() if field else None
 
     def author(self):
         """Returns the author from field 100, 110 or 111."""
-        if self["100"]:
-            return self["100"].format_field()
-        elif self["110"]:
-            return self["110"].format_field()
-        elif self["111"]:
-            return self["111"].format_field()
-        return None
+        field = self["100"] or self["110"] or self["111"]
+        return field.format_field() if field else None
 
     def uniformtitle(self):
         """Returns the uniform title from field 130 or 240."""
-        if self["130"]:
-            return self["130"].format_field()
-        elif self["240"]:
-            return self["240"].format_field()
-        return None
+        field = self["130"] or self["240"]
+        return field.format_field() if field else None
 
     def series(self):
         """Returns series fields.
@@ -547,28 +541,12 @@ class Record:
         Note: Fields 690-699 are considered "local" added entry fields but
         occur with some frequency in OCLC and RLIN records.
         """
+        # fmt: off
         return self.get_fields(
-            "600",
-            "610",
-            "611",
-            "630",
-            "648",
-            "650",
-            "651",
-            "653",
-            "654",
-            "655",
-            "656",
-            "657",
-            "658",
-            "662",
-            "690",
-            "691",
-            "696",
-            "697",
-            "698",
-            "699",
+            "600", "610", "611", "630", "648", "650", "651", "653", "654", "655",
+            "656", "657", "658", "662", "690", "691", "696", "697", "698", "699",
         )
+        # fmt: on
 
     def addedentries(self):
         """Returns Added entries fields.
@@ -576,25 +554,12 @@ class Record:
         Note: Fields 790-799 are considered "local" added entry fields but
         occur with some frequency in OCLC and RLIN records.
         """
+        # fmt: off
         return self.get_fields(
-            "700",
-            "710",
-            "711",
-            "720",
-            "730",
-            "740",
-            "752",
-            "753",
-            "754",
-            "790",
-            "791",
-            "792",
-            "793",
-            "796",
-            "797",
-            "798",
-            "799",
+            "700", "710", "711", "720", "730", "740", "752", "753", "754", "790",
+            "791", "792", "793", "796", "797", "798", "799",
         )
+        # fmt: on
 
     def location(self):
         """Returns location field (852)."""
@@ -602,66 +567,16 @@ class Record:
 
     def notes(self):
         """Return notes fields (all 5xx fields)."""
+        # fmt: off
         return self.get_fields(
-            "500",
-            "501",
-            "502",
-            "504",
-            "505",
-            "506",
-            "507",
-            "508",
-            "510",
-            "511",
-            "513",
-            "514",
-            "515",
-            "516",
-            "518",
-            "520",
-            "521",
-            "522",
-            "524",
-            "525",
-            "526",
-            "530",
-            "533",
-            "534",
-            "535",
-            "536",
-            "538",
-            "540",
-            "541",
-            "544",
-            "545",
-            "546",
-            "547",
-            "550",
-            "552",
-            "555",
-            "556",
-            "561",
-            "562",
-            "563",
-            "565",
-            "567",
-            "580",
-            "581",
-            "583",
-            "584",
-            "585",
-            "586",
-            "590",
-            "591",
-            "592",
-            "593",
-            "594",
-            "595",
-            "596",
-            "597",
-            "598",
-            "599",
+            "500", "501", "502", "504", "505", "506", "507", "508", "510", "511",
+            "513", "514", "515", "516", "518", "520", "521", "522", "524", "525",
+            "526", "530", "533", "534", "535", "536", "538", "540", "541", "544",
+            "545", "546", "547", "550", "552", "555", "556", "561", "562", "563",
+            "565", "567", "580", "581", "583", "584", "585", "586", "590", "591",
+            "592", "593", "594", "595", "596", "597", "598", "599",
         )
+        # fmt: on
 
     def physicaldescription(self):
         """Return physical description fields (300)."""
@@ -692,7 +607,7 @@ class Record:
 
 def map_marc8_record(record):
     """Map MARC-8 record."""
-    record.fields = map(map_marc8_field, record.fields)
+    record.fields = [map_marc8_field(field) for field in record.fields]
     leader = list(record.leader)
     leader[9] = "a"  # see http://www.loc.gov/marc/specifications/speccharucs.html
     record.leader = "".join(leader)
